import com.objectmentor.SocketService.SocketServer;
import com.objectmentor.SocketService.SocketService;
import junit.framework.TestCase;
import junit.swingui.TestRunner;

import java.io.*;
import java.net.Socket;

class TestSMCRServer implements SocketServer {
  public String filename = "noFileName";
  public long fileLength = -1;
  public boolean fileReceived = false;
  private ObjectOutputStream os;
  private ObjectInputStream is;
  public char[] content;
  public String[] command;

  public void serve(Socket socket) {
    try {
      os = new ObjectOutputStream(socket.getOutputStream());
      is = new ObjectInputStream(socket.getInputStream());
      os.writeObject("SMCR Test Server");
      os.flush();
      parse((String) is.readObject());
    } catch (Exception e) {
    }
  }

  private void parse(String cmd) throws Exception {
    if (cmd != null) {
      if (cmd.equals("Sending")) {
        filename = (String) is.readObject();
        fileLength = is.readLong();
        content = (char[]) is.readObject();
        fileReceived = true;
      } else if (cmd.equals("Execute")) {
        command = (String[])is.readObject();
      }
    }
  }
}

public class TestSMCRemoteClient extends TestCase {

  private static final int SMCPORT = 9000;

  public static void main(String[] args) {
    TestRunner.main(new String[]{"TestSMCRemoteClient"});
  }

  public TestSMCRemoteClient(String name) {
    super(name);
  }

  private SMCRemoteClient c;
  private TestSMCRServer server;
  private SocketService smc;

  public void setUp() throws Exception {
    c = new SMCRemoteClient();
    server = new TestSMCRServer();
    smc = new SocketService(SMCPORT, server);
  }

  public void tearDown() throws Exception {
    c.close();
    smc.close();
  }

  public void testParseCommandLine() throws Exception {
    c.parseCommandLine(new String[]{"filename"});
    assertEquals("filename", c.filename());
  }

  public void testParseInvalidCommandLine() {
    boolean result = c.parseCommandLine(new String[0]);
    assert("result should be false", !result);
  }

  public void testCountBytesInFile() throws Exception {
    File f = createTestFile("testFile", "some text");
    c.setFilename("testFile");
    boolean prepared = c.prepareFile();
    f.delete();
    assert(prepared);
    assertEquals(9, c.getFileLength());
  }

  private File createTestFile(String name, String content) throws IOException {
    File f = new File(name);
    FileOutputStream stream = new FileOutputStream(f);
    stream.write(content.getBytes());
    stream.close();
    return f;
  }

  public void testFileDoesNotExist() throws Exception {
    c.setFilename("thisFileDoesNotExist");
    boolean prepared = c.prepareFile();
    assertEquals(false, prepared);
  }

  public void testConnectToSMCRemoteServer() throws Exception {
    boolean connection = c.connect();
    assert(connection);
  }

  public void testSendFile() throws Exception {
    File f = createTestFile("testSendFile", "I am sending this file.");
    c.setFilename("testSendFile");
    assert(c.connect());
    assert(c.prepareFile());
    assert(c.sendFile());
    Thread.sleep(50);
    assert(server.fileReceived);
    assertEquals("testSendFile", server.filename);
    assertEquals(23, server.fileLength);
    assertEquals("I am sending this file.", new String(server.content));
    f.delete();
  }

  public void testExecuteCommand() throws Exception {
    assert(c.connect());
    String args[] = new String[] {"Yah","hay","dere","Eh?"};
    assert("execute",c.execute(args));
    Thread.yield();
    assertEquals("", "Yah",server.command[0]);
    assertEquals("", "hay",server.command[1]);
    assertEquals("", "dere",server.command[2]);
    assertEquals("", "Eh?",server.command[3]);
  }
}