
import java.util.ArrayList;
import java.util.List;
import java.util.Collections;
import java.io.PrintStream;

public class AsynchronousLogger implements Logger {
  private List messages = Collections.synchronizedList(new ArrayList());
  private Thread t;
  private boolean running = false;
  private int logged = 0;
  private PrintStream logStream;

  public AsynchronousLogger(PrintStream stream) {
    logStream = stream;
    running = true;
    t = new Thread(new LoggerThread());
    t.setPriority(Thread.NORM_PRIORITY - 3);
    t.start();
  }

  private void mainLoggerLoop() {
    while (running) {
      logQueuedMessages();
      sleepTillMoreMessagesQueued();
    }
  }

  private void logQueuedMessages() {
    while (messagesInQueue() > 0)
      logOneMessage();
  }

  private void logOneMessage() {
    String msg;
    msg = (String) messages.remove(0);
    logStream.println(msg);
    logged++;
  }

  private void sleepTillMoreMessagesQueued() {
    try {
      synchronized (messages) {
        messages.wait();
      }
    } catch (InterruptedException e) {
    }
  }

  public void logMessage(String msg) {
    messages.add(msg);
    wakeLoggerThread();
  }

  public int messagesInQueue() {
    return messages.size();
  }

  public int messagesLogged() {
    return logged;
  }

  public void stop() throws InterruptedException {
    running = false;
    wakeLoggerThread();
    t.join();
  }

  private void wakeLoggerThread() {
    synchronized (messages) {
      messages.notifyAll();
    }
  }

  private class LoggerThread implements Runnable {
    public void run() {
      mainLoggerLoop();
    }
  }
}
