//----------------------------------------------
//
// FSM:       TurnStile
// Context:   TurnStileContext
// Err Func:  FSMError
// Version:   
// Generated: Thursday 06/20/2002 at 14:18:27 CDT
//
//----------------------------------------------


//----------------------------------------------
//
// class TurnStile
//    This is the Finite State Machine class
//
public class TurnStile extends TurnStileContext
{
  private State itsState;
  private static String itsVersion = "";

  // instance variables for each state
  private static Locked itsLockedState;
  private static Unlocked itsUnlockedState;

  // constructor
  public TurnStile()
  {
    itsLockedState = new Locked();
    itsUnlockedState = new Unlocked();

    itsState = itsLockedState;

    // Entry functions for: Locked
  }

  // accessor functions

  public String getVersion()
  {
    return itsVersion;
  }

  public String getCurrentStateName()
  {
    return itsState.stateName();
  }

  // event functions - forward to the current State

  public void Pass()
  {
    itsState.pass();
  }

  public void Coin()
  {
    itsState.coin();
  }

  //--------------------------------------------
  //
  // private class State
  //    This is the base State class
  //
  private abstract class State
  {
    public abstract String stateName();

    // default event functions

    public void pass()
    {
      FSMError( "Pass", itsState.stateName());
    }

    public void coin()
    {
      FSMError( "Coin", itsState.stateName());
    }

  }

  //--------------------------------------------
  //
  // class Locked
  //    handles the Locked State and its events
  //
  private class Locked extends State 
  {
    public String stateName() 
      { return "Locked"; }

    //
    // responds to Pass event
    //
    public void pass() 
    {
      Alarm();

      // change the state
      itsState = itsLockedState;
    }

    //
    // responds to Coin event
    //
    public void coin() 
    {
      Unlock();

      // change the state
      itsState = itsUnlockedState;
    }
  }

  //--------------------------------------------
  //
  // class Unlocked
  //    handles the Unlocked State and its events
  //
  private class Unlocked extends State 
  {
    public String stateName() 
      { return "Unlocked"; }

    //
    // responds to Pass event
    //
    public void pass() 
    {
      Lock();

      // change the state
      itsState = itsLockedState;
    }

    //
    // responds to Coin event
    //
    public void coin() 
    {
      Thankyou();

      // change the state
      itsState = itsUnlockedState;
    }
  }

}
